/**
 * Copyright (c) 2022 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.plugin.jbiplugin;

import java.util.List;

import org.apache.maven.artifact.Artifact;
import org.apache.maven.model.Dependency;
import org.apache.maven.plugin.MojoExecutionException;
import org.apache.maven.plugin.MojoFailureException;
import org.apache.maven.plugins.annotations.Mojo;
import org.apache.maven.plugins.annotations.ResolutionScope;
import org.apache.maven.project.MavenProject;

import com.ebmwebsourcing.easycommons.lang.UncheckedException;

/**
 * Validate a JBI Archive of a project.
 * 
 * @author Christophe DENEUX - Linagora
 */
@Mojo(name = "jbi-validate", requiresDependencyResolution = ResolutionScope.RUNTIME)
public class JBIValidateMojo extends JBIAbstractConfigurableMojo {

    @Override
    public void executeMojo() throws MojoExecutionException, MojoFailureException {

        switch (this.project.getPackaging()) {
            case PACKAGING_COMPONENT:
                this.validateComponent();
                break;
            case PACKAGING_SA:
                this.validateServiceAssembly();
                break;
            case PACKAGING_SU:
                this.validateServiceUnit();
                break;
            case PACKAGING_SL:
                this.validateSharedLibrary();
                break;
            default: throw new UncheckedException("Impossible case: "+ project.getPackaging());
        }
    }

    /**
     * Validate a JBI component project
     */
    private void validateComponent() throws MojoExecutionException {

        this.info("Start validating the JBI component project " + this.projectArtifact.getArtifactId());

        // Nothing to do to validate a JBI component project

        this.info("JBI component project validated.");
    }

    /**
     * Validate a JBI service-unit project.
     */
    private void validateServiceUnit() throws MojoFailureException {

        this.info("Start validating JBI service-unit project " + this.projectArtifact.getArtifactId());
        this.checkSuAgainstJbiComponent(this.project);
        this.info("JBI service-unit project validated.");
    }

    private void checkSuAgainstJbiComponent(final MavenProject serviceUnitProject) throws MojoFailureException {

        final List<Dependency> serviceUnitDependencies = serviceUnitProject.getDependencies();
        int dependentComponentNumber = 0;
        for (final Dependency serviceUnitDependency : serviceUnitDependencies) {
            this.debug("SU dependency: " + serviceUnitDependency);
            if (PACKAGING_COMPONENT.equals(serviceUnitDependency.getType())) {
                // We have found a component
                this.debug("Component dependency found: " + serviceUnitDependency);
                dependentComponentNumber++;
            }
        }

        if (dependentComponentNumber == 0) {
            final String msg = String.format("A dependency as JBI component is required for a JBI service unit '%s'.",
                    serviceUnitProject.getArtifactId());
            this.error(msg);
            throw new MojoFailureException(msg);
        } else if (dependentComponentNumber > 1) {
            final String msg = String.format(
                    "Only one dependency as JBI component is expected for a JBI service unit '%s'.",
                    serviceUnitProject.getArtifactId());
            this.error(msg);
            throw new MojoFailureException(msg);
        }

    }

    /**
     * Validate a JBI service-assembly project.
     * 
     * @throws MojoFailureException
     */
    private void validateServiceAssembly() throws MojoExecutionException, MojoFailureException {

        this.info("Start validating the JBI service-assembly project " + this.projectArtifact.getArtifactId());

        // Check that each service unit dependency depends on only one JBI component
        final List<Dependency> dependencies = project.getDependencies();
        for (final Dependency dependency : dependencies) {

            if (PACKAGING_SU.equals(dependency.getType())) {
                try {
                    final Artifact serviceUnitArtifact = this.createDependencyArtifact(dependency);

                    this.artifactResolver.resolve(serviceUnitArtifact, this.project.getRemoteArtifactRepositories(),
                            this.localRepository);

                    final MavenProject serviceUnitProject = this.mavenProjectBuilder.buildFromRepository(
                            serviceUnitArtifact, this.project.getRemoteArtifactRepositories(), this.localRepository);

                    this.checkSuAgainstJbiComponent(serviceUnitProject);
                } catch (final MojoFailureException e) {
                    throw e;
                } catch (final Exception e) {
                    throw new MojoExecutionException(e.getLocalizedMessage(), e);
                }
            }
        }

        this.info("JBI service-assembly project validated.");
    }

    /**
     * Validate a JBI shared-library project.
     */
    private void validateSharedLibrary() throws MojoExecutionException {

        this.info("Start validating the JBI shared-library project " + this.projectArtifact.getArtifactId());

        // Nothing to do to validate a JBI shared-library project

        this.info("JBI shared-library project validated.");
    }

    @Override
    protected MavenProject getMavenProject() {
        return this.project;
    }
}
